include <DobloFactory/doblo-factory.scad>;
include <DobloFactory/lib/doblo-params.scad>;

/*
The DD play: common configuration constants and modules  
@author Perrier Stéphanie, Le Coultre Régis, Borgognon Nathalie
@date 15 dec. 2015
@licence CC BY-NC-SA 4.0
*/

// Base dimensions
$BASE = 32;
$BASE_THICKNESS = 3.5;
$BASE_DIMENSION = $BASE + ($BASE_THICKNESS * 2);

$BASE_DOBLO_DIMENSION = $BASE_DIMENSION / 8;
$BASE_DOBLO_THICKNESS = $BASE_THICKNESS / 8;
$BASE_DOBLO_HEIGHT = 12;
$BASE_NEBBLES_HEIGHT = 2;

// Vivo type
$VIVO_XL = 5;
$VIVO_L = 4;
$VIVO_M = 3;
$VIVO_S = 2;
$VIVO_XS = 1;

//bottom
$VIVO_MAX = 15.5;
$VIVO_MIN = $VIVO_MAX / $VIVO_XL;
$VIVO_XS_UP_BOTTOM = $VIVO_MIN * $VIVO_XS;
$VIVO_S_UP_BOTTOM = $VIVO_MIN * $VIVO_S;
$VIVO_M_UP_BOTTOM = $VIVO_MIN * $VIVO_M;
$VIVO_L_UP_BOTTOM = $VIVO_MIN * $VIVO_L;
$VIVO_BOTTOM_HEIGHT = $VIVO_MIN * $VIVO_XL;
$VIVO_UP_TOP = 0;
$VIVO_L_TOP_HEIGHT = $VIVO_MIN * $VIVO_L;
$VIVO_M_TOP_HEIGHT = $VIVO_MIN * $VIVO_M;
$VIVO_S_TOP_HEIGHT = $VIVO_MIN * $VIVO_S;
$VIVO_XS_TOP_HEIGHT = $VIVO_MIN * $VIVO_XS;

// TV dimensions
$TV_DOBLO_WIDTH = $BASE / 16;
$TV_DOBLO_LENGTH = $BASE / 8;
$TV_DOBLO_HEIGHT = $BASE_DOBLO_HEIGHT / 2;

// TV type
$TV_PLUS = 0;
$TV_BASIC = 1;
$TV_LIGHT = 2;
$TV_PLUS_POKA_YOKE = [2,1];
$TV_BASIC_POKA_YOKE = [1,2];


// Tel-fix dimensions
$TELFIX_DOBLO_DIMENSION = $BASE / 16;
$TELFIX_DOBLO_HEIGHT = $BASE_DOBLO_HEIGHT / 2;

// N-tel dimensions
$NTEL_DOBLO_DIMENSION = $BASE / 16;
$NTEL_DOBLO_HEIGHT = $BASE_DOBLO_HEIGHT / 2;

// Dots dimension and floor positions
$DOTS_DIAMETER = $BASE_DIMENSION / 6.65;

$TOP_LEFT_DOT = 0;
$TOP_RIGHT_DOT = 1;
$MIDDLE_LEFT_DOT = 2;
$MIDDLE_CENTER_DOT = 3;
$MIDDLE_RIGHT_DOT = 4;
$BOTTOM_LEFT_DOT = 5;
$BOTTOM_RIGHT_DOT = 6;

centerOffset = $BASE_DIMENSION / 2;
leftOffset = centerOffset / 2;
rightOffset = leftOffset * 3;
topOffset = rightOffset * -1;
middleOffset = ($BASE_DIMENSION / 2) * -1;
bottomOffset = topOffset / 3;

$DOTS_POSITION = [[leftOffset, topOffset,0], [rightOffset, topOffset,0], [leftOffset, middleOffset,0], [centerOffset, middleOffset,0], [rightOffset, middleOffset,0], [leftOffset,bottomOffset,0], [rightOffset,bottomOffset,0]];

// Number of fragments for dots
$fn = 40;

/**
 * Create one face of dots of the dice over 7 dots
 * @param topLeftDot is the point on the top left of the dice visible
 * @param topRightDot is the point on the top right of the dice visible
 * @param middleLeftDot is the point on the middle left of the dice visible
 * @param middleCenterDot is the point on the middle center of the dice visible
 * @param middleRightDot is the point on the middle right of the dice visible
 * @param bottomLeftDot is the point on the bottom left of the dice visible
 * @param bottomRightDot is the point on the bottom right of the dice visible
 */
module dotsFactory ( topLeftDot, topRightDot, middleLeftDot, middleCenterDot, middleRightDot, bottomLeftDot, bottomRightDot ) {
    
    isDotsVisible = [topLeftDot, topRightDot, middleLeftDot, middleCenterDot, middleRightDot, bottomLeftDot, bottomRightDot];
    
    for (i = [$TOP_LEFT_DOT : $BOTTOM_RIGHT_DOT]) {
        if (isDotsVisible[i] == true) {
            translate($DOTS_POSITION[i]) {
                sphere(d=$DOTS_DIAMETER);
            }
        }
    }
}

/**
 * Base cylinder to apply with an intersection to create a chamfer with the x direction over the dice
 */
module xChamfer ( ) {
    translate([-1 *($BASE_DIMENSION / 2),-1 *($BASE_DIMENSION / 2),$BASE_DIMENSION / 2]) {
        rotate(a=[0,90,0]) #cylinder(d=$BASE_DIMENSION*1.35,h=$BASE_DIMENSION * 2,center=false,$fn=360);
    }
}

/**
 * Base cylinder to apply with an intersection to create a chamfer with the y direction over the dice
 */
module yChamfer ( ) {
    translate([$BASE_DIMENSION / 2,$BASE_DIMENSION / 2,$BASE_DIMENSION / 2]) {
        rotate(a=[90,0,0]) #cylinder(d=$BASE_DIMENSION*1.35,h=$BASE_DIMENSION * 2,center=false,$fn=360);
    }
}

/**
 * Base cylinder to apply with an intersection to create a chamfer with the z direction over the dice
 */
module zChamfer ( ) {
    translate([$BASE_DIMENSION / 2,-1 *($BASE_DIMENSION / 2),-1 *($BASE_DIMENSION / 2)]) {
        rotate(a=[0,0,0]) #cylinder(d=$BASE_DIMENSION*1.35,h=$BASE_DIMENSION * 2,center=false,$fn=360);
    }
}

    
    

