include <DobloFactory/doblo-factory.scad>;
include <DobloFactory/lib/doblo-params.scad>;
include <dd.scad>;

/*
The DD play: Vivo base piece 
@author Perrier Stéphanie, Le Coultre Régis, Borgognon Nathalie
@date 15 dec. 2015
@licence CC BY-NC-SA 4.0
*/

module vivoFactory ( vivoType ) {
    intersection() {
        // base and dots faces
        difference() {
            union() {
                block (col=0, row=0, up=0, width=$BASE_DOBLO_DIMENSION, length=$BASE_DOBLO_DIMENSION, height=$BASE_THICKNESS / 2, nibbles_on_off=false, diamonds_on_off=false, scale=LUGO);
                block (col=0, row=0, up=$BASE_THICKNESS / 2, width=$BASE_DOBLO_DIMENSION, length=$BASE_DOBLO_DIMENSION, height=$BASE_DOBLO_HEIGHT + ($BASE_THICKNESS / 2), nibbles_on_off=false, diamonds_on_off=false, scale=LUGO);
            }
          
            // 3 dots dice face
            dotsFactory(topLeftDot=false, topRightDot=true, middleLeftDot=false, middleCenterDot=true, middleRightDot=false, bottomLeftDot=true, bottomRightDot=false);
            
            // 6 dots dice face
            rotate([-90,0,0]) {
                dotsFactory(topLeftDot=true, topRightDot=true, middleLeftDot=true, middleCenterDot=false, middleRightDot=true, bottomLeftDot=true, bottomRightDot=true);
            }
         
            // 1 dots dice face
            translate([0, -1 * $BASE_DIMENSION, 0]) {
                rotate([-90,0,0]) {
                    dotsFactory(topLeftDot=false, topRightDot=false, middleLeftDot=false, middleCenterDot=true, middleRightDot=false, bottomLeftDot=false, bottomRightDot=false);
                }
            }
         
            // 2 dots dice face
            rotate([-90,0,-90]) {
                dotsFactory(topLeftDot=false, topRightDot=true, middleLeftDot=false, middleCenterDot=false, middleRightDot=false, bottomLeftDot=true, bottomRightDot=false);
            }
            
            // 5 dots dice face
            translate([$BASE_DIMENSION, 0, 0]) {
                rotate([-90,0,-90]) {
                    dotsFactory(topLeftDot=true, topRightDot=true, middleLeftDot=false, middleCenterDot=true, middleRightDot=false, bottomLeftDot=true, bottomRightDot=true);
                }
            }
        }
        // Apply chamfers
        xChamfer();
        yChamfer();
        zChamfer();
    }
        
    // nebbles
    translate([$BASE_THICKNESS,-1 *$BASE_THICKNESS,($BASE_THICKNESS * 2) * 0.75]) {
        for(i = [0 : 3]) {
            if (i == 0) {
                nibbles (col=i, row=0,  up=$BASE_DOBLO_HEIGHT,  width=1, length=1, scale=LUGO);
                if (vivoType == $VIVO_XL || vivoType == $VIVO_L || vivoType == $VIVO_M) {
                    for(j = [0 : $BASE_NEBBLES_HEIGHT - 1]) {
                        nibbles (col=i, row=1,  up=$BASE_DOBLO_HEIGHT + j,  width=1, length=1, scale=LUGO);
                    }  
                    // no nebbles on col=0 && row=2
                } else if (vivoType == $VIVO_S || vivoType == $VIVO_XS) {
                    // no nebbles on col=0 && row=1
                    for(j = [0 : $BASE_NEBBLES_HEIGHT - 1]) {
                        nibbles (col=i, row=2,  up=$BASE_DOBLO_HEIGHT + j,  width=1, length=1, scale=LUGO);
                    } 
                }
                nibbles (col=i, row=3,  up=$BASE_DOBLO_HEIGHT,  width=1, length=1, scale=LUGO);
            } else if (i == 1) {
                nibbles (col=i, row=0,  up=$BASE_DOBLO_HEIGHT,  width=1, length=1, scale=LUGO); 
                if (vivoType == $VIVO_XL || vivoType == $VIVO_L || vivoType == $VIVO_M) {
                    // no nebbles on col=1 && row=1
                    for(j = [0 : $BASE_NEBBLES_HEIGHT - 1]) {
                        nibbles (col=i, row=2,  up=$BASE_DOBLO_HEIGHT + j,  width=1, length=1, scale=LUGO);
                    } 
                } else if (vivoType == $VIVO_S || vivoType == $VIVO_XS) {
                    for(j = [0 : $BASE_NEBBLES_HEIGHT - 1]) {
                        nibbles (col=i, row=1,  up=$BASE_DOBLO_HEIGHT + j,  width=1, length=1, scale=LUGO);
                    } 
                    // no nebbles on col=1 && row=2
                }
                    
                nibbles (col=i, row=3,  up=$BASE_DOBLO_HEIGHT,  width=1, length=1, scale=LUGO);
            } else {
                nibbles (col=i, row=0,  up=$BASE_DOBLO_HEIGHT,  width=1, length=4, scale=LUGO); 
            }
        }
    }
}

/**
 * Slice Vivo into a bottom part or a top part
 * @param up the up coodinates to slice
 * @param height the height dimension to slice
 */
module sliceVivo ( vivoUp, vivoHeight ) {
    block (col=-1 *$BASE_DOBLO_THICKNESS, row=-1 *$BASE_DOBLO_THICKNESS, up=vivoUp, width=$BASE_DOBLO_DIMENSION + ($BASE_DOBLO_THICKNESS * 2), length=$BASE_DOBLO_DIMENSION + ($BASE_DOBLO_THICKNESS * 2), height=vivoHeight, nibbles_on_off=false, diamonds_on_off=false, scale=LUGO);
}